/*
HomerLbc
Copyright 2009 by Jamal Mazrui
GNU Lesser General Public License (LGPL)
*/

import Homer
import Accessibility
import Microsoft.VisualBasic
import Microsoft.VisualBasic.Compatibility.VB6
import Microsoft.VisualBasic.ApplicationServices
import Microsoft.VisualBasic.Devices
import Microsoft.VisualBasic.FileIO
import Microsoft.VisualBasic.Logging
import Microsoft.VisualBasic.MyServices
import Microsoft.Win32
import System
import System.Collections
import System.Collections.ObjectModel
import System.Collections.Specialized
import System.ComponentModel
import System.Data
import System.Data.OleDb
import System.Diagnostics
import System.Drawing
import System.IO
import System.Media
import System.Net
import System.Reflection
import System.Runtime.InteropServices
import System.Text
import System.Text.RegularExpressions
import System.Threading
import System.Web
import System.Windows.Forms
import System.Xml

package Homer {
class Lbc {
static var CodeFile = ''
static var ImportFile = ''
static var ExportFile = ''

static function DialogBrowseForFolder(sTitle, sDir) {
// Get a folder path to browse

var sReturn = ''
var dlg = new FolderBrowserDialog()
if (sTitle) dlg.Description = sTitle
dlg.ShowNewFolderButton = true
dlg.SelectedPath = sDir

if (dlg.ShowDialog() == DialogResult.OK) sReturn = dlg.SelectedPath
dlg.Dispose()
return sReturn
} // BrowseForFolder method

static function DialogChoose(sTitle, aButtons, iFocus) {
// Choose a button

if (!sTitle) var sTitle = 'Choose'
if (!aButtons) var aButtons = []
if (!iFocus) var iFocus = 0
aButtons.push('Cancel')

var dlg = new LbcForm(sTitle,'this.HandleDialogChoose', null)
for (var i = 0; i < aButtons.length; i++){
var sButton = aButtons[i]
var btn = dlg.AddButton(sButton)
if ((aButtons.length - i) > 1) dlg.AddBand()
}
dlg.CompleteDialog([], iFocus)

var sReturn = dlg.Result
dlg.Dispose()
return sReturn
} // DialogChoose method

static function DialogConfirm(sTitle, sMessage, sDefault) {
// Confirm the answer to a Yes/No question

if (!sTitle) sTitle = 'Confirm'
if (!sMessage) sMessage = ''
var defaultButton = MessageBoxDefaultButton.Button1
if (sDefault.ToLower() == 'n') defaultButton = MessageBoxDefaultButton.Button2
switch (MessageBox.Show(sMessage, sTitle, MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question, defaultButton)) {
case DialogResult.Yes :
return 'Y'
case DialogResult.No :
return 'N'
}
return ''
} // DialogConfirm method

static function DialogDatePicker(sTitle, sLabel, dtValue, dtMinValue, dtMaxValue) {
// Pick a date value

if (!sTitle) var sTitle = 'DateTime Pick'
if (!sLabel) var sLabel = ''
if (!dtValue) var dtValue = null
if (!dtMinValue) var dtMinValue = null
if (!dtMaxValue) var dtMaxValue = null

var dlg = new LbcForm(sTitle,'this.HandleDialogDatePicker', null)
var dtp:DateTimePicker = dlg.AddDatePickerBox(sLabel, dtValue, dtMinValue, dtMaxValue)
dlg.CompleteDialog(null, null)

var dtReturn = dlg.Result
dlg.Dispose()
return dtReturn
} // DialogDatePicker method

static function DialogInfo(sTitle, sLabel, sText) {
// Show multiple lines of scrollable information

if (!sTitle) var sTitle = 'Info'
if (!sLabel) var sLabel = ''
if (!sText) var sText = ''

var dlg = new LbcForm(sTitle,'this.HandleDialogInfo', null)
var memo = dlg.AddMemoBox(sLabel, sText, true)
var aButtons = ['Closing']
dlg.CompleteDialog(aButtons, null)
dlg.Dispose()
return null
} // DialogInfo method

static function DialogInput(sTitle, sName, sValue) {
// Input a value

if (!sTitle) var sTitle = 'Input'
var aNames = [sName]
var aValues = [sValue]
var aResults = Lbc.DialogMultiInput(sTitle, aNames, aValues, null)
var sResult = null
if (aResults && aResults.length) sResult = aResults[0]
return sResult
} // DialogInput method

static function DialogInputAndChoose(sTitle, sName, sValue, aButtons, iFocus) {
// Input a value and choose a button

if (!sTitle) var sTitle = 'Input and Choose'
if (!aButtons) var aButtons = []
if (!sName) var sName = ''
if (!sValue) var sValue = ''
if (!iFocus) var iFocus = 0

var dlg = new LbcForm(sTitle, 'this.HandleDialogInputAndChoose', null)
dlg.AddInputBox(sName, sValue)
aButtons.push('Cancel')
dlg.CompleteDialog(aButtons, iFocus)

var aReturn = dlg.Result
dlg.Dispose()
return aReturn
} // DialogInputAndChoose method

static function DialogMemo(sTitle, sLabel, sText) {
// Edit a multiline memo

if (!sTitle) var sTitle = 'Memo'
if (!sLabel) var sLabel = ''
if (!sText) var sText = ''

var dlg = new LbcForm(sTitle,'this.HandleDialogMemo', null)
var memo = dlg.AddMemoBox(sLabel, sText, false)
dlg.CompleteDialog(null, null)
return dlg.Result
} // DialogMemo method

static function DialogMultiCheck(sTitle, sLabel, aNames, aValues, aSelect, bSort, iIndex) {
// Check multiple values

if (!sTitle) var sTitle = 'Multi Check'
if (!sLabel) var sLabel = ''
if (!aNames) var aNames = []
if (!aValues) var aValues = aNames
if (!aSelect) var aSelect = Jax.JsInitArray(false, aNames.length)
if (!bSort) var bSort = false
if (!iIndex) var iIndex = 0

var dlg = new LbcForm(sTitle,'this.HandleDialogMultiCheck', null)
dlg.IndexStatus = aValues
// var lst:CheckedListBox = dlg.AddMultiCheckBox(sLabel, aNames, aSelect, bSort, iIndex)[1]
var lst:CheckedListBox = dlg.AddMultiCheckBox(sLabel, aNames, aSelect, bSort, iIndex)
if (bSort) lst.Sorted = true
if (iIndex >= 0 && lst.Items.Count) lst.SelectedIndex = iIndex
dlg.CompleteDialog(null, null)

var aResults = dlg.Result
dlg.Dispose()
var aReturn = []
for (var i = 0; i < aResults.length; i++) {
var iIndex = aResults[i]
var oValue = aValues[iIndex]
aReturn.push(oValue)
}
return aReturn
} // DialogMultiCheck method

static function DialogMultiInput(sTitle, aNames, aValues, iFocus) {
// Input multiple values

if (!sTitle) var sTitle = 'Multi Input'
if (!aNames) var aNames = []
if (!aValues) var aValues = Jax.JsInitArray('', aNames.length)
if (!iFocus) var iFocus = 0

var dlg = new LbcForm(sTitle,'this.HandleDialogMultiInput', null)
for (var i = 0; i < aNames.length; i++){
var sName = aNames[i]
var sValue = aValues[i]
var txt:TextBox = dlg.AddInputBox(sName, sValue)
if (aNames.length - i > 1) dlg.AddBand()
}
dlg.CompleteDialog(null, iFocus)

var aReturn = dlg.Result
dlg.Dispose()
return aReturn
} // DialogMultiInput method

static function DialogMultiMaskedInput(sTitle, aNames, aValues, aMasks, iFocus) {
// Input multiple values

if (!sTitle) var sTitle = 'Multi Input'
if (!aNames) var aNames = []
if (!aValues) var aValues = Jax.JsInitArray('', aNames.length)
if (!iFocus) var iFocus = 0

var dlg = new LbcForm(sTitle,'this.HandleDialogMultiMaskedInput', null)
for (var i = 0; i < aNames.length; i++){
var sName = aNames[i]
var sValue = aValues[i]
var sMask = aMasks[i]
// var txt:MaskedTextBox = dlg.AddMaskedInputBox(sName, sValue, sMask)[1]
var txt:MaskedTextBox = dlg.AddMaskedInputBox(sName, sValue, sMask)
if (aNames.length - i > 1) dlg.AddBand()
}
dlg.CompleteDialog(null, iFocus)

var aReturn = dlg.Result
dlg.Dispose()
return aReturn
} // DialogMultiMaskedInput method

static function DialogMultiPick(sTitle, sLabel, aNames, aValues, aSelect, bSort, iIndex) {
// Pick multiple values

if (!sTitle) var sTitle = 'Multi Pick'
if (!sLabel) var sLabel = ''
if (!aNames) var aNames = []
if (!aValues) var aValues = aNames
if (!aSelect) var aSelect = Jax.JsInitArray(false, aNames.length)
if (!bSort) var bSort = false
if (!iIndex) var iIndex = 0

var dlg = new LbcForm(sTitle,'this.HandleDialogMultiPick', null)
dlg.IndexStatus = aValues
var lst:ListBox = dlg.AddMultiPickBox(sLabel, aNames, aSelect, bSort, iIndex)
if (bSort) lst.Sorted = true
if (iIndex >= 0 && lst.Items.Count) lst.SelectedIndex = iIndex
dlg.CompleteDialog(null, null)

var aResults = dlg.Result
if (!aResults) aResults = []
dlg.Dispose()
var aReturn = []
for (var i = 0; i < aResults.length; i++) {
var iIndex = aResults[i]
var oValue = aValues[iIndex]
aReturn.push(oValue)
}
return aReturn
} // DialogMultiPick method

static function DialogNumberPicker(sTitle, sText, iValue, iMinValue, iMaxValue) {
// Pick a numeric value

if (!sTitle) var sTitle = 'NumericUpDown Pick'
if (!sText) var sText = ''
if (!iValue) var iValue = null
if (!iMinValue) var iMinValue = null
if (!iMaxValue) var iMaxValue = null

var dlg = new LbcForm(sTitle,'this.HandleDialogNumberPicker', null)
var nup:NumericUpDown = dlg.AddNumericUpDown(sText, iValue, iMinValue, iMaxValue)
dlg.CompleteDialog(null, null)

var dtReturn = dlg.Result
dlg.Dispose()
return dtReturn
} // DialogNumberPicker method

static function DialogOpenFile(sTitle, sPath) {
// Get a file path to open

var sReturn = ''
var dlg = new OpenFileDialog()
if (sTitle) dlg.Title = sTitle
var sDir
if (Directory.Exists(sPath)) sDir = sPath
else {
if (File.Exists(sPath)) dlg.FileName = sPath
sDir = Path.GetDirectoryName(sPath)
if (!Directory.Exists(sDir)) sDir = Directory.GetCurrentDirectory()
}
dlg.InitialDirectory = sDir

dlg.Filter = 'All files (*.*)|*.*'
dlg.FilterIndex = 1
dlg.ValidateNames = true
dlg.CheckPathExists = true

if (dlg.ShowDialog() == DialogResult.OK) sReturn = dlg.FileName
dlg.Dispose()
return sReturn
} // OpenFile method

static function DialogOpenFolder(sTitle, sFolder) {
// Get a folder path to open

if (!sTitle) var sTitle = 'Open Folder'
if (!sFolder) var sFolter = ''

var dlg = new LbcForm(sTitle, 'this.HandleDialogOpenFolder', null)
var sLabel = 'Name'
// var txt:TextBox = dlg.AddInputBox(sLabel, sFolder)[1]
var txt:TextBox = dlg.AddInputBox(sLabel, sFolder)
txt.AutoCompleteMode = AutoCompleteMode.SuggestAppend
txt.AutoCompleteSource = AutoCompleteSource.FileSystemDirectories

var btn = dlg.AddButton('Browse')
dlg.CompleteDialog(null, null)

var sReturn = dlg.Result
dlg.Dispose()
return sReturn
} // DialogOpenFolder method

static function DialogPick(sTitle, sLabel, aNames, aValues, bSort, iIndex) {
// Pick a value

if (!sTitle) var sTitle = 'Pick'
if (!aNames) var aNames = []
if (!aValues) var aValues = aNames
if (!bSort) bSort = false
if (!iIndex) var iIndex = 0

var dlg = new LbcForm(sTitle,'this.HandleDialogPick', null)
dlg.IndexStatus = aValues
// var lst:ListBox = dlg.AddPickBox(sLabel, aNames, bSort, iIndex)[1]
var lst:ListBox = dlg.AddPickBox(sLabel, aNames, bSort, iIndex)
if (bSort) lst.Sorted = true
if (iIndex >= 0 && lst.Items.Count) lst.SelectedIndex = iIndex
dlg.CompleteDialog(null, null)

var iIndex = dlg.Result
dlg.Dispose()
if (iIndex >= 0) return aValues[iIndex]
else return null
} // DialogPick method

static function DialogPickAndChoose(sTitle, sLabel, aNames, aValues, bSort, iIndex, aButtons, iFocus) {
// Pick a value and choose a button

if (!sTitle) var sTitle = 'Pick and Choose'
if (!sLabel) var sLabel = ''
if (!aNames) var aNames = []
if (!aValues) var aValues = aNames
if (!bSort) var bSort = false
if (!iIndex) var iIndex = 0
if (!aButtons) var aButtons = []
if (!iFocus) var iFocus = 0

var dlg = new LbcForm(sTitle, 'this.HandleDialogPickAndChoose', null)
dlg.IndexStatus = aValues
dlg.AddPickBox(sLabel, aNames, bSort, iIndex)
aButtons.push('Cancel')
dlg.CompleteDialog(aButtons, iFocus)

var aResults = dlg.Result
dlg.Dispose()
var aReturn = []
if (aResults.length) {
var oValue = null
var iIndex = aResults[0]
if (iIndex >= 0) oValue = aValues[iIndex]
aReturn.push(oValue)
var sValue = aResults[1]
aReturn.push(sValue)
}
return aReturn
} // DialogPickAndChoose method

static function DialogPickInputAndChoose(sTitle, sLabel, aNames, aValues, bSort, iIndex, sName, sValue, aButtons, iFocus) {
// Pick a value, input a value, and choose a button

if (!sTitle) var sTitle = 'Pick, Input, and Choose'
if (!sLabel) var sLabel = ''
if (!aNames) var aNames = []
if (!aValues) var aValues = aNames
if (!bSort) var bSort = false
if (!iIndex) var iIndex = 0
if (!sName) var sName = ''
if (!sValue) var sValue = ''
if (!aButtons) var aButtons = 0
if (!iFocus) var iFocus = 0

var dlg = new LbcForm(sTitle, 'this.HandleDialogPickInputAndChoose', null)
dlg.IndexStatus = aValues
dlg.AddPickBox(sLabel, aNames, bSort, iIndex)
dlg.AddBand()
dlg.AddInputBox(sName, sValue)
aButtons.push('Cancel')
dlg.CompleteDialog(aButtons, iFocus)

var aResults = dlg.Result
dlg.Dispose()
var aReturn = []
var oValue = null
var iIndex = aResults[0]
if (iIndex >= 0) oValue = aValues[iIndex]
aReturn.push(oValue)
var sValue = aResults[1]
aReturn.push(sValue)
var sButton = aResults[2]
aReturn.push(sButton)
return aReturn
} // DialogPickInputAndChoose method

static function DialogSaveFile(sTitle, sPath) {
// Get a file path to save

var sReturn = ''
var sDir
var dlg = new SaveFileDialog()
if (sTitle) dlg.Title = sTitle
if (Directory.Exists(sPath)) sDir = sPath
else {
dlg.FileName = sPath
sDir = Path.GetDirectoryName(sPath)
}

if (Directory.Exists(sDir)) dlg.InitialDirectory = sDir

dlg.Filter = 'All files (*.*)|*.*'
dlg.FilterIndex = 1
dlg.CheckPathExists = true
dlg.SupportMultiDottedExtensions = true
dlg.CreatePrompt = false
dlg.ValidateNames = true
dlg.AddExtension = true
dlg.DefaultExt = 'txt'

if (dlg.ShowDialog() == DialogResult.OK) sReturn = dlg.FileName
dlg.Dispose()
return sReturn
} // SaveFile method

static function DialogShow(oTitle, oMessage) {
// Show two values in the title and message of a message box

var sTitle = ''
if (oTitle) sTitle = oTitle.ToString()
var sMessage = ''
if (oMessage) sMessage = oMessage.ToString()
MessageBox.Show(sMessage, sTitle)
} //DialogShow method

static function DialogTimePicker(sTitle, sLabel, dtValue, dtMinValue, dtMaxValue) {
// Pick a time value

if (!sTitle) var sTitle = 'DateTime Pick'
if (!sLabel) var sLabel = ''
if (!dtValue) var dtValue = null
if (!dtMinValue) var dtMinValue = null
if (!dtMaxValue) var dtMaxValue = null

var dlg = new LbcForm(sTitle,'this.HandleDialogTimePicker', null)
var dtp:DateTimePicker = dlg.AddTimePickerBox(sLabel, dtValue, dtMinValue, dtMaxValue)
dlg.CompleteDialog(null, null)

var dtReturn = dlg.Result
dlg.Dispose()
return dtReturn
} // DialogTimePicker method

static function Init() {
// Initialize values for code, import, and export files

var aArgs = Environment.GetCommandLineArgs()
if (aArgs.Length > 1 && File.Exists(aArgs[1])) Lbc.CodeFile = aArgs[1]
if (aArgs.Length > 2) Lbc.ImportFile = aArgs[2]
if (aArgs.Length > 3) Lbc.ExportFile = aArgs[3]
} // Init method

static function StringToKey(sKey:String):Keys {
// Convert a string to a Keys object

return TypeDescriptor.GetConverter(Keys).ConvertFromString(sKey)
} // String2Key method
} // Lbc class

class LbcForm extends Form {
var Dialog = false
var MainPanel = null
var Names:OrderedDictionary = new OrderedDictionary()
// var Names = new OrderedDictionary()
var FocusStatus = {}
var IndexStatus = []
var Handler = 'OnEvent'
var Result = null
var FocusJump = ''

function LbcForm() {
// Instantiate an LbcForm

this(null, null, null)
} // LbcForm parameterless constructor

function LbcForm(sTitle, sHandler, dFocusStatus) {
// Instantiate an LbcForm, passing values for its title, event handler, and status messages

this.Init(sTitle, sHandler, dFocusStatus)
} // LbcForm constructor

function Init(sTitle, sHandler, dFocusStatus) {
// Initialize an LbcForm

this.SuspendLayout()
if (!sTitle) var sTitle = ''
if (!sHandler) var sHandler = null
if (!dFocusStatus) var dFocusStatus = null
this.Text = sTitle
if (sHandler) this.Handler = sHandler
if (dFocusStatus) this.FocusStatus = dFocusStatus

this.MainMenuStrip = CreateMainMenu()
this.Controls.Add(this.MainMenuStrip)
this.MainPanel = CreateVerticalPanel()
this.MainPanel.Dock = DockStyle.Fill
this.Controls.Add(this.MainPanel)
this.AddBand()
} // Init method

function AddBackgroundWorker() {
// Add a BackgroundWorker

var bkw:BackgroundWorker = new BackgroundWorker()
bkw.WorkerReportsProgress = true
bkw.WorkerSupportsCancellation = true
var sName = 'BackgroundWorker_'
// Does not have this property
// bkw.Name = sName
bkw.add_DoWork(Delegate.CreateDelegate(DoWorkEventHandler, this, 'HandleDoWork'))
bkw.add_ProgressChanged(Delegate.CreateDelegate(ProgressChangedEventHandler, this, 'HandleProgressChanged'))
bkw.add_RunWorkerCompleted(Delegate.CreateDelegate(RunWorkerCompletedEventHandler, this, 'HandleRunWorkerCompleted'))
return bkw
} // AddBackgroundWorker method

function AddBand() {
// Start a new band of controls

var flp = CreateHorizontalPanel()
this.MainPanel.Controls.Add(flp)
return flp
} // AddBand method

function AddButton(sText):Button {
// Add a button

var btn:Button = new Button()
if (sText == 'OK') this.AcceptButton = btn
else if (sText == 'Cancel' || sText == 'Closing') this.CancelButton = btn
else var sText = MakeCaption(sText)
btn.Text = sText
btn.AccessibleName = MakeAccessibleName(sText)
var sName = MakeName(btn, sText)
btn.Name = sName
this.Names[sName] = btn
btn.add_Click(Delegate.CreateDelegate(EventHandler, this, 'HandleClick'))
// btn.add_Click(new EventHandler(this.HandleClick))
btn.add_GotFocus(Delegate.CreateDelegate(EventHandler, this, 'HandleGotFocus'))
this.MainPanel.Controls(CurrentBand()).Controls.Add(btn)
return btn
} // AddButton method

function AddButtonBand(aButtons) {
// Add a centered band of buttons

var aReturn = []
if (!aButtons) var aButtons = ['OK', 'Cancel']
var flp = AddBand()
flp.AnchorStyles = AnchorStyles.None
for (var i = 0; i < aButtons.length; i++) {
var sButton = aButtons[i]
var btn = AddButton(sButton)
// if (i == iFocus) btn.Select()
aReturn.push(btn)
}
return aReturn
} // AddButtonBand method

function AddCheckBox(sText, bState):CheckBox {
// Add a checkbox

if (!sText) var sText = ''
if (!bState) var bState = false

var chk:CheckBox = new CheckBox()
chk.Checked = bState
var sText = MakeCaption(sText)
chk.Text = sText
chk.AccessibleName = MakeAccessibleName(sText)
var sName = MakeName(chk, sText)
chk.Name = sName
this.Names[sName] = chk
chk.add_Click(Delegate.CreateDelegate(EventHandler, this, 'HandleClick'))
chk.add_GotFocus(Delegate.CreateDelegate(EventHandler, this, 'HandleGotFocus'))
this.MainPanel.Controls(CurrentBand()).Controls.Add(chk)
return chk
} // AddCheckBox method

function AddCheckedListBox(aItems, aSelect, bSort, iIndex):CheckedListBox {
// Add a checked listbox

if (!aItems) var aItems = []
if (!aSelect) var aSelect = Jax.JsInitArray(false, aItems.length)
if (!bSort) var bSort = false
if (!iIndex) var iIndex = 0

var lst:CheckedListBox = new CheckedListBox()
// lst.SelectionMode = SelectionMode.MultiSimple
for (var i = 0; i < aItems.length; i++) {
lst.Items.Add(aItems[i])
Support.SetItemData(lst, i, i)
var bState = aSelect[i]
if (!bState) bState = false
lst.SetItemChecked(i, bState)
}

lst.Sorted = bSort
if (iIndex >= 0 && lst.Items.Count) lst.SelectedIndex = iIndex

var sName = 'CheckedListBox_'
var lbl = CurrentControl()
// if (lbl) {
if (lbl != null) {
lst.AccessibleName = lbl.AccessibleName
sName = lbl.Name.replace('Label_', 'CheckedListBox_')
}
lst.Name = sName
this.Names[sName] = lst
lst.add_GotFocus(Delegate.CreateDelegate(EventHandler, this, 'HandleGotFocus'))
lst.add_SelectedIndexChanged(Delegate.CreateDelegate(EventHandler, this, 'HandleSelectedIndexChanged'))
this.MainPanel.Controls(CurrentBand()).Controls.Add(lst)
return lst
} // AddCheckedListBox method

function AddComboBox(aItems, bSort, iIndex):ComboBox {
// Add a combobox

if (!aItems) var aItems = []
if (!bSort) var bSort = false
if (!iIndex) var iIndex = 0

var lst:ComboBox = new ComboBox()
for (var i = 0; i < aItems.length; i++) {
lst.Items.Add(aItems[i])
Support.SetItemData(lst, i, i)
}
lst.Sorted = bSort
if (iIndex >= 0 && lst.Items.Count) lst.SelectedIndex = iIndex

var sName = 'ComboBox_'
var lbl = CurrentControl()
// if (lbl) {
if (lbl != null) {
lst.AccessibleName = lbl.AccessibleName
sName = lbl.Name.replace('Label_', 'ComboBox_')
}
lst.Name = sName
this.Names[sName] = lst
lst.add_GotFocus(Delegate.CreateDelegate(EventHandler, this, 'HandleGotFocus'))
lst.add_SelectedIndexChanged(Delegate.CreateDelegate(EventHandler, this, 'HandleSelectedIndexChanged'))
this.MainPanel.Controls(CurrentBand()).Controls.Add(lst)
return lst
} // AddComboBox method

function AddComboPickBox(sLabel, aItems, bSort, iIndex):ComboBox {
// Add a labeled combobox

var lbl:Label = null
if (sLabel) lbl = AddLabel(sLabel)
var lst:ComboBox = AddComboBox(aItems, bSort, iIndex)
// return [lbl, lst]
return lst
} // AddComboPickBox method

function AddDatePicker(dtValue, dtMinValue, dtMaxValue):DateTimePicker {
// Add a date picker

if (!dtValue) var dtValue = null
if (!dtMinValue) var dtMinValue = null
if (!dtMaxValue) var dtMaxValue = null

var dtp: DateTimePicker = new DateTimePicker()
// dtp.MinDate = new DateTime(1985, 6, 20)
// dtp.MaxDate = DateTime.Today
if (dtValue) dtp.Value = dtValue
if (dtMinValue) dtp.MinDate = dtMinValue
if (dtMaxValue) dtp.MaxDate = dtMaxValue
// dtp.Format = DateTimePickerFormat.Custom
// dtp.CustomFormat = "MMMM dd, yyyy - dddd"
// dtp.Format = DateTimePickerFormat.Short
dtp.Format = DateTimePickerFormat.Long

dtp.ShowCheckBox = false
dtp.ShowUpDown = true
var sName = 'DatePicker_'
var lbl = CurrentControl()
if (lbl != null) {
dtp.AccessibleName = lbl.AccessibleName
sName = lbl.Name.replace('Label_', 'DatePicker_')
}
dtp.Name = sName
this.Names[sName] = dtp
dtp.add_GotFocus(Delegate.CreateDelegate(EventHandler, this, 'HandleGotFocus'))
this.MainPanel.Controls(CurrentBand()).Controls.Add(dtp)
return dtp
} // AddDatePicker method

function AddDatePickerBox(sLabel, dtValue, dtMinValue, dtMaxValue):DateTimePicker {
// Add a labeled date picker

if (!sLabel) var sLabel = ''
if (!dtValue) var dtValue = null
if (!dtMinValue) var dtMinValue = null
if (!dtMaxValue) var dtMaxValue = null

var lbl:Label = null
if (sLabel) lbl = AddLabel(sLabel)
var dtp:DateTimePicker = AddDatePicker(dtValue, dtMinValue, dtMaxValue)
return dtp
} // AddDatePickerBox method

function AddInputBox(sLabel, sText):TextBox {
// Add a labeled TextBox

if (!sLabel) var sLabel = ''
if (!sText) var sText = ''
var lbl:Label = null
if (sLabel) lbl = AddLabel(sLabel)
var txt:TextBox = AddTextBox(sText)
if (lbl && lbl.Name.EndsWith('Password')) txt.UseSystemPasswordChar = true
// return [lbl, txt]
return txt
} // AddInputBox method

function AddLabel(sText):Label {
// Add a label

var lbl:Label = new Label()
var sText = MakeLabel(sText)
lbl.Text = sText
lbl.AccessibleName = MakeAccessibleName(sText)
var sName = MakeName(lbl, sText)
lbl.Name = sName
this.Names[sName] = lbl
this.MainPanel.Controls(CurrentBand()).Controls.Add(lbl)
return lbl
} // AddLabel method

function AddListBox(aItems, bSort, iIndex):ListBox {
// Add a listbox

if (!aItems) var aItems = []
if (!bSort) var bSort = false
if (!iIndex) var iIndex = 0

var lst:ListBox = new ListBox()
lst.SelectionMode = SelectionMode.One
for (var i = 0; i < aItems.length; i++) {
lst.Items.Add(aItems[i])
Support.SetItemData(lst, i, i)
}
lst.Sorted = bSort
if (iIndex >= 0 && lst.Items.Count) lst.SelectedIndex = iIndex

var sName = 'ListBox_'
var lbl = CurrentControl()
// if (lbl) {
if (lbl != null) {
lst.AccessibleName = lbl.AccessibleName
sName = lbl.Name.replace('Label_', 'ListBox_')
}
lst.Name = sName
this.Names[sName] = lst
lst.add_GotFocus(Delegate.CreateDelegate(EventHandler, this, 'HandleGotFocus'))
lst.add_SelectedIndexChanged(Delegate.CreateDelegate(EventHandler, this, 'HandleSelectedIndexChanged'))
this.MainPanel.Controls(CurrentBand()).Controls.Add(lst)
return lst
} // AddListBox method

function AddMaskedInputBox(sLabel, sText, sMask):MaskedTextBox {
// Add a labeled MaskedTextBox

if (!sLabel) var sLabel = ''
if (!sText) var sText = ''
var lbl:Label = null
if (sLabel) lbl = AddLabel(sLabel)
var txt:MaskedTextBox = AddMaskedTextBox(sText, sMask)
// return [lbl, txt]
return txt
} // AddMaskedInputBox method

function AddMaskedTextBox(sText, sMask):MaskedTextBox {
// Add a MaskedTextBox

if (!sText) var sText = ''

var txt:MaskedTextBox = new MaskedTextBox()
var iWidth = txt.Width
txt.Width = 2 * iWidth
txt.Text = sText
txt.Mask = sMask
txt.BeepOnError = true

var sName = 'MaskedTextBox_'
var lbl = CurrentControl()
if (lbl != null) {
txt.AccessibleName = lbl.AccessibleName
sName = lbl.Name.replace('Label_', 'MaskedTextBox_')
}
txt.Name = sName
this.Names[sName] = txt
txt.add_GotFocus(Delegate.CreateDelegate(EventHandler, this, 'HandleGotFocus'))
// txt.add_SelectionChanged(Delegate.CreateDelegate(EventHandler, this, 'HandleSelectionChanged'))
this.MainPanel.Controls(CurrentBand()).Controls.Add(txt)
return txt
} // AddMaskedTextBox method

function AddMemo(sText, bReadOnly):TextBox {
// Add a multiline memo

if (!sText) var sText = ''
if (!bReadOnly) var bReadOnly = false

// var txt:TextBox = new TextBox()
// var txt:RichTextBox = new RichTextBox()
var txt:TextBox = new TextBox()
txt.Multiline = true
var iWidth = txt.Width
var iHeight = txt.Height
// txt.Width = 4 * iWidth
// txt.Height = 4 * iHeight
txt.Width = 300
txt.Height = 300
txt.Text = sText
txt.ReadOnly = bReadOnly
// No such property in RichTextBox
if (!bReadOnly) txt.AcceptsReturn = true
// txt.ScrollBars = RichTextBoxScrollBars.Vertical | RichTextBoxScrollBars.Horizontal
txt.ScrollBars = ScrollBars.Vertical
// Next two lines are actually defaults so not necessary
txt.AcceptsTab = false
txt.WordWrap = true

var sName = 'Memo_'
var lbl = CurrentControl()
if (lbl != null) {
txt.AccessibleName = lbl.AccessibleName
sName = lbl.Name.replace('Label_', 'Memo_')
}
txt.Name = sName
this.Names[sName] = txt
txt.add_GotFocus(Delegate.CreateDelegate(EventHandler, this, 'HandleGotFocus'))
// Only in RichTextBox
// txt.add_SelectionChanged(Delegate.CreateDelegate(EventHandler, this, 'HandleSelectionChanged'))
this.MainPanel.Controls(CurrentBand()).Controls.Add(txt)
return txt
} // AddMemo method

function AddMemoBox(sLabel, sText, bReadOnly):TextBox {
// Add a labeled memo

if (!sLabel) var sLabel = ''
if (!sText) var sText = ''
if (!bReadOnly) var bReadOnly = false

var lbl:Label = null
if (sLabel) lbl = AddLabel(sLabel)
// var txt:RichTextBox = AddMemo(sText, bReadOnly)
var txt:TextBox = AddMemo(sText, bReadOnly)
// return [lbl, txt]
return txt
} // AddMemoBox method

function AddMenu(sText):ToolStripMenuItem {
// Add a menu pulldown

var menu = CreateMenu(sText)
this.MainMenuStrip.Items.Add(menu)
return menu
} // AddMenu method

function AddMenuItem(sText, oKeys):ToolStripMenuItem {
// Add a menu item and accelerator key

var menu = CurrentMenu()
var menuItem = CreateMenuItem(sText, oKeys)
menu.DropDownItems.Add(menuItem)
return menuItem
} // AddMenuItem method

function AddMultiCheckBox(sLabel, aItems, aSelect, bSort, iIndex):CheckedListBox {
// Add a labeled checked listbox

if (!sLabel) var sLabel = ''
if (!aItems) var aItems = []
if (!aSelect) var aSelect = Jax.JsInitArray(false, aItems.length)
if (!bSort) var bSort = false
if (!iIndex) var iIndex = 0

var lbl:Label = null
if (sLabel) lbl = AddLabel(sLabel)
var lst:CheckedListBox = AddCheckedListBox(aItems, aSelect, bSort, iIndex)
// return [lbl, lst]
return lst
} // AddMultiCheckBox method

function AddMultiListBox(aItems, aSelect, bSort, iIndex):ListBox {
// Add a multiple selection listbox

if (!aItems) var aItems = []
if (!aSelect) var aSelect = Jax.JsInitArray(false, aItems.length)
if (!bSort) var bSort = false
if (!iIndex) var iIndex = 0

var lst:ListBox = new ListBox()
lst.SelectionMode = SelectionMode.MultiSimple
for (var i = 0; i < aItems.length; i++) {
lst.Items.Add(aItems[i])
Support.SetItemData(lst, i, i)
continue
var bState = aSelect[i]
if (!bState) bState = false
lst.SetSelected(i, bState)
}

lst.Sorted = bSort
if (iIndex >= 0 && lst.Items.Count) lst.SelectedIndex = iIndex

var sName = 'MultiListBox_'
var lbl = CurrentControl()
// if (lbl) {
if (lbl != null) {
lst.AccessibleName = lbl.AccessibleName
sName = lbl.Name.replace('Label_', 'MultiListBox_')
}
lst.Name = sName
this.Names[sName] = lst
lst.add_GotFocus(Delegate.CreateDelegate(EventHandler, this, 'HandleGotFocus'))
// Does not work as expected for this type of control, use MultiCheck instead
// lst.add_SelectedIndexChanged(Delegate.CreateDelegate(EventHandler, this, 'HandleSelectedIndexChanged'))
this.MainPanel.Controls(CurrentBand()).Controls.Add(lst)
return lst
} // AddMultiListBox method

function AddMultiPickBox(sLabel, aItems, aSelect, bSort, iIndex):ListBox {
// Add a labeled multiple selection listbox

if (!sLabel) var sLabel = ''
if (!aItems) var aItems = []
if (!aSelect) var aSelect = Jax.JsInitArray(false, aItems.length)
if (!bSort) var bSort = false
if (!iIndex) var iIndex = 0

var lbl:Label = null
if (sLabel) lbl = AddLabel(sLabel)
var lst:ListBox = AddMultiListBox(aItems, aSelect, bSort, iIndex)
// return [lbl, lst]
return lst
} // AddMultiPickBox method

function AddNumericUpDown(sText, iValue, iMinValue, iMaxValue):NumericUpDown {
// Add a NumericUpDown control

if (!sText) var sText = ''
if (!iValue) var iValue = null
if (!iMinValue) var iMinValue = null
if (!iMaxValue) var iMaxValue = null

var nup: NumericUpDown = new NumericUpDown()
if (iValue) nup.Value = iValue
if (iMinValue) nup.Minimum = iMinValue
if (iMaxValue) nup.Maximum = iMaxValue
var sName = 'NumericUpDown_'
nup.AccessibleName = MakeAccessibleName(sText)
var sName = MakeName(nup, sText)
nup.Name = sName
this.Names[sName] = nup
nup.add_GotFocus(Delegate.CreateDelegate(EventHandler, this, 'HandleGotFocus'))
this.MainPanel.Controls(CurrentBand()).Controls.Add(nup)
return nup
} // AddNumericUpDown method

function AddPickBox(sLabel, aItems, bSort, iIndex):ListBox {
// Add a labeled listbox

if (!sLabel) var sLabel = ''
if (!aItems) var aItems = []
if (!bSort) var bSort = false
if (!iIndex) var iIndex = 0

var lbl:Label = null
if (sLabel) lbl = AddLabel(sLabel)
var lst:ListBox = AddListBox(aItems, bSort, iIndex)
// return [lbl, lst]
return lst
} // AddPickBox method

function AddProgressBar(iMinValue, iMaxValue, iStep):ProgressBar {
// Add a progress bar

if (!iMinValue) var iMinValue = 0
if (!iMaxValue) var iMaxValue = 100
if (!iStep) var iStep = 1

var pb: ProgressBar = new ProgressBar()
pb.Minimum = iMinValue
pb.Maximum = iMaxValue
pb.Step = 1
pb.Value = pb.Minimum
var sName = 'ProgressBar_'
pb.AccessibleName = ''
pb.Name = sName
this.Names[sName] = pb
// pb.add_GotFocus(Delegate.CreateDelegate(EventHandler, this, 'HandleGotFocus'))
this.MainPanel.Controls(CurrentBand()).Controls.Add(pb)
return pb
} // AddProgressBar method



function AddRadioButton(sText, bState):RadioButton {
// Add a RadioButton

if (!sText) var sText = ''
if (!bState) var bState = false

var rbn:RadioButton = new RadioButton()
rbn.Checked = bState
var sText = MakeCaption(sText)
rbn.Text = sText
rbn.AccessibleName = MakeAccessibleName(sText)
var sName = MakeName(rbn, sText)
rbn.Name = sName
this.Names[sName] = rbn
rbn.add_Click(Delegate.CreateDelegate(EventHandler, this, 'HandleClick'))
rbn.add_GotFocus(Delegate.CreateDelegate(EventHandler, this, 'HandleGotFocus'))
this.MainPanel.Controls(CurrentBand()).Controls.Add(rbn)
return rbn
} // AddRadioButton method

function AddStatus(sText) {
// Append status text

var sStatus = sText
var sOldStatus = GetStatus()
if (!sOldStatus) sStatus = '   ' + sStatus
return SetStatus(sStatus)
} // AddStatus method

function AddStatusBar():StatusStrip {
// Add a status bar

var sb:StatusStrip = new StatusStrip()
sb.AccessibleRole = AccessibleRole.StatusBar
sb.Dock = DockStyle.Bottom
sb.AutoSize = true
// sb.AutoSizeMode = AutoSizeMode.GrowAndShrink
var lblStatus = new ToolStripStatusLabel('Ready')
lblStatus.AutoSize = true
lblStatus.AutoSizeMode = AutoSizeMode.GrowAndShrink
sb.Items.Add(lblStatus)
this.Controls.Add(sb)
return sb
} // AddStatusBar method

function AddTextBox(sText):TextBox {
// Add a textbox

if (!sText) var sText = ''

var txt:TextBox = new TextBox()
var iWidth = txt.Width
txt.Width = 2 * iWidth
txt.Text = sText
var sName = 'TextBox_'
var lbl = CurrentControl()
if (lbl != null) {
txt.AccessibleName = lbl.AccessibleName
sName = lbl.Name.replace('Label_', 'TextBox_')
}
txt.Name = sName
this.Names[sName] = txt
txt.add_GotFocus(Delegate.CreateDelegate(EventHandler, this, 'HandleGotFocus'))
// txt.add_SelectionChanged(Delegate.CreateDelegate(EventHandler, this, 'HandleSelectionChanged'))
this.MainPanel.Controls(CurrentBand()).Controls.Add(txt)
return txt
} // AddTextBox method

function AddTimePicker(dtValue, dtMinValue, dtMaxValue):DateTimePicker {
// Add a time picker

if (!dtValue) var dtValue = null
if (!dtMinValue) var dtMinValue = null
if (!dtMaxValue) var dtMaxValue = null

var dtp: DateTimePicker = new DateTimePicker()
// dtp.MinDate = new DateTime(1985, 6, 20)
// dtp.MaxDate = DateTime.Today
if (dtValue) dtp.Value = dtValue
if (dtMinValue) dtp.MinDate = dtMinValue
if (dtMaxValue) dtp.MaxDate = dtMaxValue
dtp.Format = DateTimePickerFormat.Time

dtp.ShowCheckBox = false
dtp.ShowUpDown = true
var sName = 'TimePicker_'
var lbl = CurrentControl()
if (lbl != null) {
dtp.AccessibleName = lbl.AccessibleName
sName = lbl.Name.replace('Label_', 'TimePicker_')
}
dtp.Name = sName
this.Names[sName] = dtp
dtp.add_GotFocus(Delegate.CreateDelegate(EventHandler, this, 'HandleGotFocus'))
this.MainPanel.Controls(CurrentBand()).Controls.Add(dtp)
return dtp
} // AddTimePicker method

function AddTimePickerBox(sLabel, dtValue, dtMinValue, dtMaxValue):DateTimePicker {
// Add a labeled time picker

if (!sLabel) var sLabel = ''
if (!dtValue) var dtValue = null
if (!dtMinValue) var dtMinValue = null
if (!dtMaxValue) var dtMaxValue = null

var lbl:Label = null
if (sLabel) lbl = AddLabel(sLabel)
var dtp:DateTimePicker = AddTimePicker(dtValue, dtMinValue, dtMaxValue)
return dtp
} // AddTimePickerBox method

function CollectionToArray(oCollection) {
// Convert a collection to an array

var aReturn = []
for (var o in oCollection) aReturn.push(o)
return aReturn
} // CollectionToArray method

function CompleteDialog(aButtons, iFocus):Button[] {
this.Dialog = true
return Complete(aButtons, iFocus)
} // CompleteDialog method

function CompleteWindow(aButtons, iFocus):Button[] {
this.Dialog = false
return Complete(aButtons, iFocus)
} // CompleteWindow method

function Complete(aButtons, iFocus):Button[] {
// Complete a dialog with a button band and focus position

if (!aButtons || aButtons.length) {
var a = AddButtonBand(aButtons)
this.AcceptButton = a[0]
}
if (!iFocus) var iFocus = 0

AddStatusBar()
this.add_Load(Delegate.CreateDelegate(EventHandler, this, 'HandleLoad'))
this.add_Shown(Delegate.CreateDelegate(EventHandler, this, 'HandleShown'))
this.add_Closing(Delegate.CreateDelegate(CancelEventHandler, this, 'HandleClosing'))
// this.Controls.Add(this.MainMenuStrip)
this.KeyPreview = true
this.add_KeyDown(Delegate.CreateDelegate(KeyEventHandler, this, 'HandleKeyDown'))
this.StartPosition = FormStartPosition.CenterScreen
this.AutoSize = true
this.AutoSizeMode = AutoSizeMode.GrowAndShrink
// this.AutoScroll = true
var aItems = CollectionToArray(this.Names)
// Start at 1 not 0
for (var i = 1; i < this.Names.Count; i++) {
if (i == iFocus) {
// var ctl = this.Names[i]
// var ctl = this.Names.Item(i)
var ctl = aItems[i].Value
// ctl.Select()
ctl.Focus()
break
}
}
this.ResumeLayout()
if (this.Dialog) this.ShowDialog()
else {
this.Show()
this.Activate()
}
} // Complete method

function CreateFlowPanel(direction) {
// Create a FlowLayoutPanel

var flp = new FlowLayoutPanel()
flp.FlowDirection = direction
flp.AutoSize = true
flp.AutoSizeMode = AutoSizeMode.GrowAndShrink
return flp
} // CreateFlowPanel method

function CreateHorizontalPanel() {
// Create a horizontal FlowLayoutPanel

return CreateFlowPanel(FlowDirection.TopDown)
} // CreateHorizontalPanel method

function CreateMainMenu():MenuStrip {
// Create the main menu of the dialog

var menuMain:MenuStrip = new MenuStrip()
menuMain.AccessibleRole = AccessibleRole.MenuBar
menuMain.Dock = DockStyle.Top
/*
menuMain.AutoSize = true
menuMain.CanOverflow = false
menuMain.LayoutStyle = ToolStripLayoutStyle.Flow
menuMain.Stretch = false
*/
return menuMain
} // CreateMainMenu method

function CreateMenu(sText) {
// Create a menu pulldown

var menu = new ToolStripMenuItem(sText)
menu.AccessibleRole = AccessibleRole.MenuItem
menu.Text = sText
menu.Name = sText
return menu
} // CreateMenu method

function CreateMenuItem(sText, sKey) {
// Create a menu item

if (!sText) var sText = ''
if (!sKey) var sKey = ''
var oImage = null
var oHandler = Delegate.CreateDelegate(EventHandler, this, 'HandleClick')
var menuItem = new ToolStripMenuItem(sText, , oHandler,)
menuItem.AccessibleRole = AccessibleRole.MenuItem
menuItem.ShowKeyboardCues = true
menuItem.ShowShortcutKeys = true
menuItem.Text = sText
var oKey = Lbc.StringToKey(sKey)
menuItem.ShortcutKeys = oKey
var sFriendlyKey = MakeFriendlyKeyName(sKey)
menuItem.ShortcutKeyDisplayString = sFriendlyKey
var sName = MakeAccessibleName(sText)
sName = sName.Replace(' ', '_')
menuItem.AccessibleName = sName + '   ' + sFriendlyKey
sName = 'MenuItem_' + sName
menuItem.Name = sName
this.Names[sName] = menuItem

/*
menuItem.Tag = sOptions
menuItem.ToolTipText = sKey
menuItem.AccessibleDescription = Util.GetFriendlyKeyName(sKey)
menuItem.ShortcutKeyDisplay= sFriendlyKey
*/
return menuItem
} // CreateMenuItem method

function CreateTablePanel(iColumns, iRows) {
// Create a TableLayoutPanel

var tlp = new TableLayoutPanel()
tlp.Anchor = AnchorStyles.None
tlp.AutoSize = true
tlp.AutoSizeMode = AutoSizeMode.GrowAndShrink
//tlp.AutoScroll = true
tlp.ColumnCount = iColumns
for (var i = 0; i < tlp.ColumnCount; i++) {
tlp.ColumnStyles.Add(new ColumnStyle(SizeType.AutoSize))
}

for (var i = 0; i < tlp.RowCount; i++) {
tlp.RowStyles.Add(new RowStyle(SizeType.AutoSize))
}
return tlp
} // CreateTablePanel method

function CreateVerticalPanel() {
// Create a vertical FlowLayoutPanel

return CreateFlowPanel(FlowDirection.TopDown)
} // CreateVerticalPanel method

function CurrentBand() {
// Return the current band number

return this.MainPanel.Controls.Count - 1
} // CurrentBand method

function CurrentControl() {
// Return the control last added

var iBand = CurrentBand()
if (iBand < 0) return null

var flp = this.MainPanel.Controls(iBand)
var iIndex = flp.Controls.Count - 1
if (iIndex < 0) return null

var ctl = flp.Controls(iIndex)
return ctl
} // CurrentControl method

function CurrentMenu() {
// Return the menu pulldown last added

if (this.MainMenuStrip == null) return null
var iIndex = this.MainMenuStrip.Items.Count - 1
if (iIndex < 0) return null
var menu = this.MainMenuStrip.Items(iIndex)
return menu
} // CurrentMenu method

function CurrentMenuItem() {
// Return the menu item last added

var menu = CurrentMenu()
if (menu == null) return null

var iIndex = menu.DropDownItems.Count - 1
if (iIndex < 0) return null
var menuItem = menu.DropDownItems(iIndex)
return menuItem
} // CurrentMenuItem method

function Export(sFile) {
// Export data from a file into controls of the current dialog

var aItems = CollectionToArray(this.Names)
for (var i = 0; i < aItems.length; i++) {
var oItem = aItems[i]
var sName = oItem.Key
if (sName.StartsWith('MenuItem_')) continue
var ctl = oItem.Value
if (ctl.Items) {
var sText = ''
for (var j = 0; j < ctl.Items.Count; j++) {
if (sText.length) sText += '\n'
sText += ctl.Items(j).ToString()
} // j
}
else sText = ctl.Text
var s = this.Text.Replace('&', '').Replace(' ', '_')
var sPath = 'Dialog_' + s + '/' + sName
var oJax = new Jax()
oJax.XmlSetValue(sFile, sPath, sText)
} // i
} // Export method

function FindControl(sClass) {
// Find the first control with a class name

var aItems = CollectionToArray(this.Names)
for (var i = 0; i < this.Names.Count; i++) {
// var ctl = this.Names[i]
// var ctl = this.Names.Item(i)
var ctl = aItems[i].Value
if (ctl.Name.StartsWith(sClass + '_')) return ctl
}
return null
} // FindControl method

function GetClass(ctl) {
// Get the class of a control

var sType = ctl.GetType().ToString()
var aParts = sType.split('.')
var sClass = aParts.slice(-1)
return sClass
} // GetClass method

function GetFocus() {
// Get the control with focus

var aItems = CollectionToArray(this.Names)
for (var i = 0; i < this.Names.Count; i++) {
// var ctl = this.Names[i]
// var ctl = this.Names.Item(i)
var ctl = aItems[i].Value
if (ctl.Focused) return ctl
}
return null
} // GetFocus method

function GetStatus() {
// Get status text

var iIndex = this.Controls.Count - 1
var sb:StatusStrip = this.Controls(iIndex)
var lbl = sb.Items(0)
return lbl.Text
} // GetStatus method

function HandleClick(oSender:Object, oArgs:EventArgs) {
// Handler for Click event

var sEvent = 'Click'
// var sCode = this.Handler + '(sEvent, oSender, oArgs)'
// eval(sCode, 'unsafe')
var bHandler = false
try {
if (typeof(eval(this.Handler, 'unsafe')) == 'function') bHandler = true
}
catch (e) {}

var bResult = false
if (bHandler) {
var sCode = this.Handler + '(sEvent, oSender, oArgs)'
bResult = eval(sCode, 'unsafe')
}
if (bResult) return
} // HandleClickEvent method

function HandleClosing(oSender:Object, oArgs:CancelEventArgs) {
// Handler for Closing event of a dialog

var sEvent = 'Closing'
var bHandler = false
try {
if (typeof(eval(this.Handler, 'unsafe')) == 'function') bHandler = true
}
catch (e) {}

var bResult = false
if (bHandler) {
var sCode = this.Handler + '(sEvent, oSender, oArgs)'
bResult = eval(sCode, 'unsafe')
}
if (bResult) return

if (!File.Exists(Lbc.ExportFile)) return
this.Export(Lbc.ExportFile)
} // HandleClosing method

function HandleDialogChoose(sEvent, oSender, oArgs) {
// Custom event handler for Choose dialog

switch (sEvent) {
case 'Load' :
this.Result = 'Cancel'
return
case 'Click' :
this.Result = oSender.AccessibleName
this.Close()
} // sEvent
} // HandleDialogChoose method

function HandleDialogDatePicker(sEvent, oSender, oArgs) {
// Custom event handler for date picker dialog

var dtp = FindControl('DatePicker')
switch (sEvent) {
case 'Load' :
this.Result = null
return
case 'Click' :
switch(oSender.Name) {
case 'Button_OK' :
this.Result = dtp.Value
} // oSender.Name
this.Close()
} // sEvent
} // HandleDialogDatePicker method

function HandleDialogInfo(sEvent, oSender, oArgs) {
// Custom event handler for Info dialog

switch (sEvent) {
case 'Click' :
this.Close()
return
} // sEvent
} // HandleDialogInfo method

function HandleDialogInputAndChoose(sEvent, oSender, oArgs) {
// Custom event handler for InputAndChoose dialog

switch (sEvent) {
case 'Load' :
this.Result = []
return
case 'Click' :
var aResults = []
var txt = FindControl('TextBox')
aResults.push(txt.Text)
aResults.push(oSender.AccessibleName)
this.Result = aResults
this.Close()
} // sEvent
} // HandleDialogInputAndChoose method

function HandleDialogMemo(sEvent, oSender, oArgs) {
// Custom event handler for Memo dialog

switch (sEvent) {
case 'Click' :
this.Result = ''
switch(oSender.Name) {
case 'Button_OK' :
var txt = FindControl('Memo')
this.Result = txt.Text
break
} // oSender.Name
this.Close()
} // sEvent
} // HandleDialogMemo method

function HandleDialogMultiCheck(sEvent, oSender, oArgs) {
// Custom event handler for multi check dialog

switch (sEvent) {
case 'Load' :
this.Result = []
return
case 'Index' :
var lst = FindControl('CheckedListBox')
var iIndex = lst.SelectedIndex
if (iIndex < 0) return
var iData = Support.GetItemData(lst, iIndex)
var sValue = this.IndexStatus[iData]
if (sValue) SetStatus(sValue)
return
case 'Click' :
switch (oSender.Name) {
case 'Button_OK' :
var lst = FindControl('CheckedListBox')
var aResults = []
for (var i = 0; i < lst.CheckedIndices.Count; i++) {
var iIndex = lst.CheckedIndices(i)
var iData = Support.GetItemData(lst, iIndex)
aResults.push(iData)
}
this.Result = aResults
} // oSender.Name
this.Close()
} // sEvent
} // HandleDialogMultiCheck method

function HandleDialogMultiInput(sEvent, oSender, oArgs) {
// Custom event handler for MultiInput dialog

switch (sEvent) {
case 'Load' :
this.Result = []
return
case 'Click' :
switch (oSender.Name) {
case 'Button_OK' :
var aResults = []
for (var i = 0; i < this.MainPanel.Controls.Count; i++) {
var flp = this.MainPanel.Controls(i)
for (var j = 0; j < flp.Controls.Count; j++) {
var ctl = flp.Controls(j)
if (ctl.Name.StartsWith('TextBox_')) aResults.push(ctl.Text)
}
}
this.Result = aResults
} // oSender.Name
this.Close()
} // sEvent
} // HandleDialogMultiInput method

function HandleDialogMultiMaskedInput(sEvent, oSender, oArgs) {
// Custom event handler for MultiMaskedInput dialog

switch (sEvent) {
case 'Load' :
this.Result = []
return
case 'Click' :
switch (oSender.Name) {
case 'Button_OK' :
var aResults = []
for (var i = 0; i < this.MainPanel.Controls.Count; i++) {
var flp = this.MainPanel.Controls(i)
for (var j = 0; j < flp.Controls.Count; j++) {
var ctl = flp.Controls(j)
if (ctl.Name.StartsWith('MaskedTextBox_')) aResults.push(ctl.Text)
}
}
this.Result = aResults
} // oSender.Name
this.Close()
} // sEvent
} // HandleDialogMultiMaskedInput method

function HandleDialogMultiPick(sEvent, oSender, oArgs) {
// Custom event handler for multi pick dialog

switch (sEvent) {
case 'Load' :
this.Result = []
return
case 'Index' :
var lst = FindControl('MultiListBox')
var iIndex = lst.SelectedIndex
if (iIndex < 0) return
var iData = Support.GetItemData(lst, iIndex)
var sValue = this.IndexStatus[iData]
if (sValue) SetStatus(sValue)
return
case 'Click' :
switch (oSender.Name) {
case 'Button_OK' :
var lst = FindControl('MultiListBox')
var aResults = []
for (var i = 0; i < lst.SelectedIndices.Count; i++) {
var iIndex = lst.SelectedIndices(i)
var iData = Support.GetItemData(lst, iIndex)
aResults.push(iData)
}
this.Result = aResults
} // oSender.Name
this.Close()
} // sEvent
} // HandleDialogMultiPick method

function HandleDialogNumberPicker(sEvent, oSender, oArgs) {
// Custom event handler for NumberPicker dialog

var nup = FindControl('NumericUpDown')
switch (sEvent) {
case 'Load' :
this.Result = null
return
case 'Click' :
switch(oSender.Name) {
case 'Button_OK' :
this.Result = nup.Value
} // oSender.Name
this.Close()
} // sEvent
} // HandleDialogNumberPicker method

function HandleDialogOpenFolder(sEvent, oSender, oArgs) {
// Custom event handler for OpenFolder dialog

var txt = FindControl('TextBox')
var sFolder = txt.Text.Trim()
switch (sEvent) {
case 'Load' :
this.Result = ''
return
case 'Click' :
switch (oSender.Name) {
case 'Button_Browse' :
sFolder = Lbc.DialogBrowseForFolder('', sFolder)
if (sFolder) txt.Text = sFolder
txt.Select()
txt.SelectAll()
return
case 'Button_OK' :
if (Directory.Exists(sFolder)) {
this.Result = sFolder
break
}

var sMessage = 'Cannot find folder\n' + sFolder + '\nCreate it?'
if (Lbc.DialogConfirm('', sMessage, 'Y') != 'Y') return

try {
Directory.CreateDirectory(sFolder)
this.Result = sFolder
break
}
catch (e) {
Lbc.DialogShow('Error', e.Description)
return
}
} // oSender.Name
this.Close()
} // sEvent
} // HandleDialogOpenFolder method

function HandleDialogPick(sEvent, oSender, oArgs) {
// Custom event handler for Pick dialog

var lst = FindControl('ListBox')
var iIndex = lst.SelectedIndex
switch (sEvent) {
case 'Load' :
this.Result = -1
return
case 'Index' :
if (iIndex < 0) return
var iData = Support.GetItemData(lst, iIndex)
var sValue = this.IndexStatus[iData]
if (sValue) SetStatus(sValue)
return
case 'Click' :
switch(oSender.Name) {
case 'Button_OK' :
if (iIndex >= 0) this.Result = Support.GetItemData(lst, iIndex)
} // oSender.Name
this.Close()
} // sEvent
} // HandleDialogPick method

function HandleDialogPickAndChoose(sEvent, oSender, oArgs) {
// Custom event handler for PickAndChoose dialog

switch (sEvent) {
case 'Load' :
this.Result = []
return
case 'Index' :
var lst = FindControl('ListBox')
var iIndex = lst.SelectedIndex
if (iIndex < 0) return
var iData = Support.GetItemData(lst, iIndex)
var sValue = this.IndexStatus[iData]
if (sValue) SetStatus(sValue)
return
case 'Click' :
var aResults = []
var lst = FindControl('ListBox')
var iIndex = lst.SelectedIndex
var iData = -1
if (iIndex >= 0) iData = Support.GetItemData(lst, iIndex)
aResults.push(iData)
aResults.push(oSender.AccessibleName)
this.Result = aResults
this.Close()
} // sEvent
} // HandleDialogPickAndChoose method

function HandleDialogPickInputAndChoose(sEvent, oSender, oArgs) {
// Custom event handler for PickInputAndChoose dialog

switch (sEvent) {
case 'Load' :
this.Result = []
return
case 'Index' :
var lst = FindControl('ListBox')
var iIndex = lst.SelectedIndex
if (iIndex < 0) return
var iData = Support.GetItemData(lst, iIndex)
var sValue = this.IndexStatus[iData]
if (sValue) SetStatus(sValue)
return
case 'Click' :
var aResults = []
var lst = FindControl('ListBox')
var iIndex = lst.SelectedIndex
var iData = -1
if (iIndex >= 0) iData = Support.GetItemData(lst, iIndex)
aResults.push(iData)
var txt = FindControl('TextBox')
aResults.push(txt.Text)
aResults.push(oSender.AccessibleName)
this.Result = aResults
this.Close()
} // sEvent
} // HandleDialogPickInputAndChoose method

function HandleDialogTimePicker(sEvent, oSender, oArgs) {
// Custom event handler for time picker dialog

var dtp = FindControl('TimePicker')
switch (sEvent) {
case 'Load' :
this.Result = null
return
case 'Click' :
switch(oSender.Name) {
case 'Button_OK' :
this.Result = dtp.Value
} // oSender.Name
this.Close()
} // sEvent
} // HandleDialogTimePicker method

function HandleDoWork(oSender:Object, oArgs:DoWorkEventArgs) {
// Handler for DoWork event

var sEvent = 'DoWork'
// var sCode = this.Handler + '(sEvent, oSender, oArgs)'
// eval(sCode, 'unsafe')
var bHandler = false
try {
if (typeof(eval(this.Handler, 'unsafe')) == 'function') bHandler = true
}
catch (e) {}

var bResult = false
if (bHandler) {
var sCode = this.Handler + '(sEvent, oSender, oArgs)'
bResult = eval(sCode, 'unsafe')
}
if (bResult) return
} // HandleDoWorkEvent method


function HandleGotFocus(oSender:Object, oArgs:EventArgs) {
// Handler for GotFocus event of a control

var sEvent = 'GotFocus'
var bHandler = false
try {
if (typeof(eval(this.Handler, 'unsafe')) == 'function') bHandler = true
}
catch (e) {}

var bResult = false
if (bHandler) {
var sCode = this.Handler + '(sEvent, oSender, oArgs)'
bResult = eval(sCode, 'unsafe')
}
if (bResult) return

var sText = this.FocusStatus[oSender.Name]
if (!sText) sText = ''
SetStatus(sText)
} // HandleGotFocusEvent method

function HandleKeyDown(oSender:Object, oArgs:KeyEventArgs) {
// Handler for KeyDown event

var sEvent = 'KeyDown'
var bHandler = false
try {
if (typeof(eval(this.Handler, 'unsafe')) == 'function') bHandler = true
}
catch (e) {}

var bResult = false
if (bHandler) {
var sCode = this.Handler + '(sEvent, oSender, oArgs)'
bResult = eval(sCode, 'unsafe')
}
if (bResult) return

var oKey = oArgs.KeyData
switch(oKey) {
case Keys.Control | Keys.F9 :
case Keys.Alt | Keys.F9 :
var sJump = this.FocusJump
var bAgain = true
if (oKey == (Keys.Control | Keys.F9)) {
var sJump = Lbc.DialogInput('Focus Jump', 'Text', sJump)
if (!sJump) return
if (sJump != this.FocusJump) {
bAgain = false
this.FocusJump = sJump
}
}

var iFocus = -1
var bFound = false
var aItems = CollectionToArray(this.Names)
for (var j = 0; j < 2; j++) {
for (var i = 0; i < this.Names.Count; i++) {
// var ctl = this.Names[i]
// var ctl = this.Names.Item(i)
var ctl = aItems[i].Value
if (iFocus == -1 && j == 0 && !ctl.Focused) continue
if (iFocus == -1) iFocus = i

if (!ctl.CanFocus) continue
var sClass = GetClass(ctl)
if (sClass == 'ListBox') {
var iStart = ctl.SelectedIndex
if (bAgain) iStart++
if (j == 1) iStart = 0
if (iStart == -1) iStart = 0

for (var iIndex = iStart; iIndex < ctl.Items.Count; iIndex++) {
if (!ctl.Items(iIndex).ToString().ToLower().Contains(sJump.ToLower())) continue
bFound = true
ctl.Focus()
ctl.SelectedIndex = iIndex
break
}
if (bFound) break
else continue
} // ListBox

var sText = ctl.Text
if (!sText) continue

if (sClass == 'TextBox') {
var iStart = ctl.SelectionStart
if (bAgain) iStart++
if (j == 1) iStart = 0
var iIndex = sText.ToLower().IndexOf(sJump.ToLower(), iStart)
if (iIndex == -1) continue

bFound = true
ctl.Focus()
ctl.SelectionStart = iIndex
break
}

if (j == 0 && bAgain && i == iFocus) continue
if (!sText.ToLower().Contains(sJump.ToLower())) continue
bFound = true
// if (sClass == 'Label') ctl.SelectNextControl()
// if (sClass == 'Label') ctl = this.Names.Item(i + 1)
if (sClass == 'Label') ctl = aItems[i + 1].Value
ctl.Focus()
break
} // i
if (bFound) break
} // j}
return
case (Keys.Control | Keys.F8) :
var ctl = GetFocus()
var sClass = GetClass(ctl)
if (sClass == 'ListBox') {
}

if (sClass == 'TextBox') {
}
if (sClass == 'Label') {
}
return
} // F8
} // HandleKeyDown method

function HandleLoad(oSender:Object, oArgs:EventArgs) {
// Handler for Load event

var sEvent = 'Load'
var bHandler = false
try {
if (typeof(eval(this.Handler, 'unsafe')) == 'function') bHandler = true
}
catch (e) {}

var bResult = false
if (bHandler) {
var sCode = this.Handler + '(sEvent, oSender, oArgs)'
bResult = eval(sCode, 'unsafe')
}
if (bResult) return

if (!File.Exists(Lbc.ImportFile)) return
this.Import(Lbc.ImportFile)
} // HandleLoadEvent method

function HandleProgressChanged(oSender:Object, oArgs:ProgressChangedEventArgs) {
// Handler for ProgressChanged event

var sEvent = 'ProgressChanged'
// var sCode = this.Handler + '(sEvent, oSender, oArgs)'
// eval(sCode, 'unsafe')
var bHandler = false
try {
if (typeof(eval(this.Handler, 'unsafe')) == 'function') bHandler = true
}
catch (e) {}

var bResult = false
if (bHandler) {
var sCode = this.Handler + '(sEvent, oSender, oArgs)'
bResult = eval(sCode, 'unsafe')
}
if (bResult) return
} // HandleProgressChangedEvent method


function HandleRunWorkerCompleted(oSender:Object, oArgs:RunWorkerCompletedEventArgs) {
// Handler for RunWorkerCompleted event

var sEvent = 'RunWorkerCompleted'
// var sCode = this.Handler + '(sEvent, oSender, oArgs)'
// eval(sCode, 'unsafe')
var bHandler = false
try {
if (typeof(eval(this.Handler, 'unsafe')) == 'function') bHandler = true
}
catch (e) {}

var bResult = false
if (bHandler) {
var sCode = this.Handler + '(sEvent, oSender, oArgs)'
bResult = eval(sCode, 'unsafe')
}
if (bResult) return
} // HandleRunWorkerCompletedEvent method


function HandleSelectedIndexChanged(oSender:Object, oArgs:EventArgs) {
// Handler for SelectedIndexChanged event of a ListBox

var sEvent = 'Index'
var bHandler = false
try {
if (typeof(eval(this.Handler, 'unsafe')) == 'function') bHandler = true
}
catch (e) {}

var bResult = false
if (bHandler) {
var sCode = this.Handler + '(sEvent, oSender, oArgs)'
bResult = eval(sCode, 'unsafe')
}
if (bResult) return
} // HandleSelectedIndexChangedEvent method

function HandleSelectionChanged(oSender:Object, oArgs:EventArgs) {
// Handler for SelectionChanged event of a RichTextBox

var sEvent = 'SelectionChanged'
var bHandler = false
try {
if (typeof(eval(this.Handler, 'unsafe')) == 'function') bHandler = true
}
catch (e) {}

var bResult = false
if (bHandler) {
var sCode = this.Handler + '(sEvent, oSender, oArgs)'
bResult = eval(sCode, 'unsafe')
}
if (bResult) return
} // HandleSelectionChangedEvent method

function HandleShown(oSender:Object, oArgs:EventArgs) {
// Handle for Shown event of a dialog

var sEvent = 'Shown'
var bHandler = false
try {
if (typeof(eval(this.Handler, 'unsafe')) == 'function') bHandler = true
}
catch (e) {}

var bResult = false
if (bHandler) {
var sCode = this.Handler + '(sEvent, oSender, oArgs)'
bResult = eval(sCode, 'unsafe')
}
if (bResult) return
} // HandleShownEvent method

function Import(sFile) {
// Import data from a file into controls of the current dialog

var oDoc = Jax.XmlOpenFile(sFile)
var sName = 'Dialog_' + this.Text
sName = sName.Replace('&', '').Replace(' ', '_')
var oNodes = oDoc.selectNodes('Root/' + sName + '/*')
for (var i = 0; i < oNodes.length; i++) {
var oNode = oNodes.item(i)
var sName = oNode.nodeName
if (sName.StartsWith('MenuItem_')) continue
var ctl = this.Names[sName]
if (!ctl) continue

var sText = oNode.text
if (ctl.Items) {
var aLines = sText.split('\n')
ctl.Items.Clear()
for (var j = 0; j < aLines.length; j++) ctl.Items.Add(aLines[j], j)
if (ctl.Items.Count) ctl.SelectedIndex = 0
}
else ctl.Text = sText
} // i

oNodes = null
} // Import method

function MakeAccessibleName(sLabel) {
// Make an accessible control name

var sName = sLabel.Replace('&', '')
sName = sName.Replace(':', '')
return sName
} // MakeAccessibleName method

function MakeCaption(sText) {
// Make a control caption

var sReturn = sText
if (sText.IndexOf('&') == -1) sReturn = '&' + sText
return sReturn
} // MakeCaption method

function MakeFriendlyKeyName(sKey) {
// Make a key string more friendly

if (sKey.Contains('+OemQuotes')) sKey = sKey.Replace('+OemQuotes', '+Apostrophe')
if (sKey.Contains('+Back')) sKey = sKey.Replace('+Back', '+Backspace')
if (sKey.Contains('+Oem5')) sKey = sKey.Replace('+Oem5', '+Backslash')
if (sKey.Contains('+Oemplus')) sKey = sKey.Replace('+Oemplus', '+Equals')
if (sKey.Contains('+OemMinus')) sKey = sKey.Replace('+OemMinus', '+Dash')
if (sKey.Contains('+OemSemicolon')) sKey = sKey.Replace('+OemSemicolon', '+Semicolon')
if (sKey.Contains('+D6')) sKey = sKey.Replace('+D6', '+Caret')
if (sKey.Contains('+D0')) sKey = sKey.Replace('+D0', '+0')
if (sKey.Contains('+OemQuestion')) sKey = sKey.Replace('+OemQuestion', '+Slash')
if (sKey.Contains('+OemOpenBrackets')) sKey = sKey.Replace('+OemOpenBrackets', '+LeftBracket')
if (sKey.Contains('+OemCloseBrackets')) sKey = sKey.Replace('+OemCloseBrackets', '+RightBracket')
if (sKey.Contains('+Oemcomma')) sKey = sKey.Replace('+Oemcomma', '+Comma')
if (sKey.Contains('+OemPeriod')) sKey = sKey.Replace('+OemPeriod', '+Period')
return sKey
} // MakeFriendlyKeyName method

function MakeLabel(sText) {
// Make a control label

var sReturn = MakeCaption(sText)
if (sReturn.indexOf(':') == -1) sReturn += ':'
return sReturn
} // MakeLabel method

function MakeName(ctl, sLabel) {
// Make a control name

var sType = GetClass(ctl)
var sName = MakeAccessibleName(sLabel)
sName = sName.Replace(' ', '_')
sName = sType + '_' + sName
return sName
} // MakeName method

function SetStatus(sText) {
// Set status text

var iIndex = this.Controls.Count - 1
var sb:StatusStrip = this.Controls(iIndex)
var lbl = sb.Items(0)
var sOldStatus = lbl.Text
lbl.Text = sText
return sOldStatus
} // SetStatus method
} // LbcForm class
class LbcProgressBarWindow extends LbcForm {
function LbcProgressBarWindow() {
this(null, null, null, null)
} // LbcProgressBarWindow constructor

function LbcProgressBarWindow(sTitle, iMinValue, iMaxValue, iStep) {

if (!sTitle) var sTitle = 'Progress Bar'
if (!iMinValue) var iMinValue = 0
if (!iMaxValue) var iMaxValue = 100
if (!iStep) var iStep = 1

this.Init(sTitle, 'this.HandleProgressBarWindow', null)
var pb:ProgressBar = this.AddProgressBar(iMinValue, iMaxValue, iStep)
this.CompleteWindow(['Closing'], null)
} // ProgressBarWindow constructor

function HandleProgressBarWindow(sEvent, oSender, oArgs) {
// Custom event handler for ProgressBarWindow

switch (sEvent) {
case 'Load' :
this.Result = true
return
case 'Click' :
switch(oSender.Name) {
case 'Button_Close' :
// this.Dispose()
this.Result = false
this.Close()
return
} // oSender.Name
} // sEvent
} // HandleProgressBarWindow method

function GetValue():Decimal {
var pb = this.Names['ProgressBar_']
return pb.Value
} // GetValue method

function SetValue(nValue) {
var pb = this.Names['ProgressBar_']
pb.Value = nValue
} // SetValue method

function Step() {
var pb = this.Names['ProgressBar_']
pb.PerformStep()
} // Step method
} // LbcProgressBarWindow class

} // Homer package
