/*
HomerZip
Copyright 2009 by Jamal Mazrui
GNU Lesser General Public License (LGPL)
*/

using Homer;
using Accessibility;
using Microsoft.VisualBasic;
using Microsoft.VisualBasic.Compatibility.VB6;
using Microsoft.VisualBasic.ApplicationServices;
using Microsoft.VisualBasic.Devices;
using Microsoft.VisualBasic.FileIO;
using Microsoft.VisualBasic.Logging;
using Microsoft.VisualBasic.MyServices;
using Microsoft.Win32;
using System;
using System.Collections;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Data;
using System.Data.OleDb;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Media;
using System.Net;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Web;
using System.Windows.Forms;
using System.Xml;

namespace Homer {
public class Zip {
static string Dir = "";
static string Exe = "";
static string Password = "";

static Zip() {
Zip.Dir = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location);
Zip.Exe = Path.Combine(Zip.Dir, "7z.exe");
} // Zip constructor

public static void Init(string sExe, string sPassword) {
// Initialize static properties for 7z.exe archive path and password

Zip.Exe = sExe;
Zip.Password = sPassword;
} // Init method

public static bool IsOK(string sText) {
// Test wehter 7.exe output indicates successful result

sText = sText.Replace("\r\n", "\n");
string sMatch = "\nEverything is Ok\n";
return sText.IndexOf(sMatch) >= 0;
} // IsOK method

static string GetListFile(string[] aFiles) {
string sFile = "";
if (aFiles != null && aFiles.Length > 0) {
sFile = Path.GetTempFileName();
string sFiles = String.Join("\r\n", aFiles);
File.WriteAllText(sFile, sFiles);
}
return sFile;
} // GetListFile method

public static bool Test(string sZip) {
// Test integrity of archive

string sParams = "t " + Zip.Quote(sZip);
if (Zip.Password.Length > 0) sParams += " -p" + Zip.Password;
string sText = Zip.GetProgramOutput(Zip.Exe, sParams);
return IsOK(sText);
} // Test method

public static bool Add(string sZip, string[] aFiles) {
// Add array of files to archive (or null for * wildcard)

string sFile = GetListFile(aFiles);
string sParams = "a " + Zip.Quote(sZip);
if (sFile.Length > 0) sParams += " " + "@" + Zip.Quote(sFile);
if (Zip.Password.Length > 0) sParams += " -p" + Zip.Password;
string sText = Zip.GetProgramOutput(Zip.Exe, sParams);
// Clipboard.SetText(Zip.Exe + " " + sParams);
// Console.WriteLine(sText);
if (File.Exists(sFile)) File.Delete(sFile);
return IsOK(sText);
} // Add method

public static bool Delete(string sZip, string[] aFiles) {
// Delete array of files from archive (or null for * wildcard)

string sFile = GetListFile(aFiles);
string sParams = "d " + Zip.Quote(sZip);
if (sFile.Length > 0) sParams += " " + "@" + Zip.Quote(sFile);
if (Zip.Password.Length > 0) sParams += " -p" + Zip.Password;
string sText = Zip.GetProgramOutput(Zip.Exe, sParams);
if (File.Exists(sFile)) File.Delete(sFile);
return IsOK(sText);
} // Delete method

public static bool Extract(string sZip, string[] aFiles, string sDir, bool bSubdirs) {
// Extract array of files from archive (or null for * wildcard)

string sFile = GetListFile(aFiles);
string sParams = "e ";
if (bSubdirs) sParams = "x ";
sParams += Zip.Quote(sZip);
if (sFile.Length > 0) sParams += " " + "@" + Zip.Quote(sFile);
if (sDir != null && sDir.Length > 0) sParams += " " + Zip.Quote("-o" + sDir);
sParams += " -aoa";
if (Zip.Password.Length > 0) sParams += " -p" + Zip.Password;
string sText = Zip.GetProgramOutput(Zip.Exe, sParams);
if (File.Exists(sFile)) File.Delete(sFile);
return IsOK(sText);
} // Extract method

public static string[] GetZipContents(string sZip) {
// Get output of 7z.exe command to list file properties

string sParams = "l " + Zip.Quote(sZip) + " -slt";
if (Zip.Password.Length > 0) sParams += " -p" + Zip.Password;
// Clipboard.SetText(Zip.Exe + " " + sParams);
// Console.WriteLine(sText);
// string sText = "";
string sText = Zip.GetProgramOutput(Zip.Exe, sParams);
sText = sText.Replace("\r\n", "\n");
sText = sText.Replace(@"\", "/");
string sMatch = @"^(.|\n)*?\n----------\n\n";
string sReplace = "";
sText = Regex.Replace(sText, sMatch, sReplace);
string[] aLines = sText.Split('\n');
return aLines;
} // GetZipContents method


public static object[] Contents(string sZip) {
// Get array of dictionaries of archive contents

ArrayList l = new ArrayList();
DateTime dt;
long iSize;
string sPath, sAttributes;

string[] aLines = GetZipContents(sZip);
sPath = "";
iSize = 0;
dt = DateTime.MinValue;
sAttributes = "";

int iCount = aLines.Length;
// Console.WriteLine("Count " + iCount);
for (int i = 0; i < iCount; i++) {
// Console.WriteLine("i " + i);
string sLine = aLines[i];
if (sPath.Length > 0 && (i > 0 && sLine.StartsWith("Path =")) || (i == iCount - 1)) {
Hashtable d = new Hashtable();
d.Add("Path", sPath);
d.Add("Size", iSize);
d.Add("Time", dt);
d.Add("Attributes", sAttributes);
l.Add(d);
sPath = "";
}

if (sLine.Length == 0) {
// Do nothing
}
else if (sLine.StartsWith("Path =")) sPath = sLine.Substring(6).Trim();
else if (sLine.StartsWith("Size =")) {
string s = sLine.Substring(6).Trim();
if (s.Length > 0) iSize = Int32.Parse(s);
else iSize = 0;
}
else if (sLine.StartsWith("Modified =")) {
if (!DateTime.TryParse(sLine.Substring(10).Trim(), out dt)) dt = DateTime.MinValue;
}
else if (sLine.StartsWith("Attributes =")) {
sAttributes = sLine.Substring(12).Trim();
sAttributes = sAttributes.Replace(".", "");
}
}
return l.ToArray();
} // Contents method

public static string Quote(string sText) {
// Quote a string of text

return "\"" + Unquote(sText) + "\"";
} // Quote method

public static string Unquote(string sText) {
// Unqoute a string of text

return sText.Trim('"');
} // Unquote method

public static string GetProgramOutput(string sExe, string sParams) {
// Get standard output from running console executable

Process process = new Process();
ProcessStartInfo startInfo = new ProcessStartInfo(sExe, sParams);
startInfo.UseShellExecute = false;
//startInfo.RedirectStandardInput = true;
startInfo.RedirectStandardOutput = true;
startInfo.RedirectStandardError = true;
startInfo.WorkingDirectory = Path.GetDirectoryName(sExe);
startInfo.ErrorDialog = true;
startInfo.CreateNoWindow = true;
startInfo.WindowStyle = ProcessWindowStyle.Hidden;
process.StartInfo = startInfo;
process.Start();
StreamReader stream = process.StandardOutput;
process.WaitForExit();
string sText = stream.ReadToEnd();
stream.Close();
process.Close();
return sText;
} // GetProgramOutput method
} // Zip class
} // Homer namespace
